/**
 * 
 */
package com.objectbay.switchyard.example;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.exporter.ZipExporter;
import org.jboss.shrinkwrap.api.spec.EnterpriseArchive;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.shrinkwrap.resolver.api.DependencyResolvers;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependencyResolver;

/**
 * @author Christoph Gostner <christoph.gostner@objectbay.com>
 * 
 */
public class EarBuilder {
	private MavenDependencyResolver resolver;
	private EnterpriseArchive ear;
	private ModuleAdder moduleAdder;
	private LibAdder libAdder;
	private List<String> moduleNames;
	private String name;

	public EarBuilder(String archiveName) {
		this.ear = ShrinkWrap.create(EnterpriseArchive.class, archiveName);
		this.name = archiveName;
		moduleNames = new ArrayList<String>();
		
		this.moduleAdder = new ModuleAdder();
		this.libAdder = new LibAdder();
	}

	public void loadPom(String pomXml) {
		this.resolver = DependencyResolvers.use(MavenDependencyResolver.class).loadMetadataFromPom(pomXml);
	}

	public void addArtifactAndDependenciesAsLibrary(String artifact, Class<? extends Archive<?>> clazz) {
		Collection<? extends Archive<?>> libraries = resolver.artifact(artifact).resolveAs(clazz);
		ear.addAsLibraries(libraries);
	}

	public void addArtifactAsLibrary(String string, Class<? extends Archive<?>> clazz) {
		addArchive(string, libAdder, clazz);
	}

	public void addArtifactAsModule(String string, Class<? extends Archive<?>> clazz) {
		String moduleName = addArchive(string, moduleAdder, clazz);
		moduleNames.add(moduleName);
	}

	public void addAsModule(JavaArchive archive) {
		moduleAdder.add(ear, archive);
		String moduleName = archive.getName();
		moduleNames.add(moduleName);
	}

	public void export(String path) {
		ear.as(ZipExporter.class).exportTo(new File(path), true);
	}

	public void generateApplicationXml() throws IOException {
		File temp = File.createTempFile("pattern", ".suffix");
		temp.deleteOnExit();
		createApplicationXml(temp);
		ear.addAsManifestResource(temp, "application.xml");
	}
	
	public EnterpriseArchive getEar() {
		return ear;
	}

	private void createApplicationXml(File file) throws IOException {
		StringBuilder builder = new StringBuilder();
		builder.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		builder.append("<!DOCTYPE application PUBLIC \n" + 
				"\t\"-//Sun Microsystems, Inc.//DTD J2EE Application 1.3//EN\" \n" + 
				"\t\"http://java.sun.com/dtd/application_1_3.dtd\">");
		builder.append("<application>\n");
		builder.append("<display-name>").append(name).append("</display-name>");
		for (String moduleName : moduleNames) {
			builder.append("<module>\n");
			builder.append("\t<ejb>").append(moduleName).append("</ejb>\n");
			builder.append("</module>");
		}
		builder.append("</application>");

		BufferedWriter out = new BufferedWriter(new FileWriter(file));
		out.write(builder.toString());
		out.close();
	}

	private String addArchive(String artifact, Adder adder, Class<? extends Archive<?>> clazz) {
		Collection<? extends Archive<?>> jars = resolver.artifact(artifact).resolveAs(clazz);
		String name = artifact.split(":")[1];

		for (Archive<?> javaArchive : jars) {

			if (javaArchive.getName().startsWith(name)) {
				adder.add(ear, javaArchive);
				return javaArchive.getName();
			}
		}
		return null;
	}

	static interface Adder {
		void add(EnterpriseArchive ear, Archive<?>... archives);
	}

	static class LibAdder implements Adder {
		public void add(EnterpriseArchive ear, Archive<?>... archives) {
			ear.addAsLibraries(archives);
		}
	}

	static class ModuleAdder implements Adder {
		public void add(EnterpriseArchive ear, Archive<?>... archives) {
			for (Archive<?> javaArchive : archives) {
				ear.addAsModule(javaArchive);
			}
		}
	}
}
